/*
 * (c) 2016, SWD Embedded Systems Limited, http://www.kpda.ru
 */

/*
 * smarttest.c
 *
 * Elographics touch screen  protocol. 
 *
 */

#include "devi.h"
#include <time.h>
#include <pthread.h>

#define	TRUE					1
#define	FALSE					!TRUE
#define DEBOUNCE			 	80000L

#define TRY_LIMIT             (100000)
#define TIME_DELAY             (10000)
#define IN_BUFF_FULL             (0x2)
#define OUT_BUFF_FULL            (0x1)

#define AUX_IRQ                    12 /* default aux irq */
#define PS2_DATA_PORT            0x60
#define PS2_STATUS_REGISTER      0x64
#define PS2_COMMAND_REGISTER     0x64

#define PS2_TEST_CONTROLLER      0xAA
#define PS2_SELFTEST_SUCCESS     0x55
#define PS2_READ_CONFBYTE        0x20
#define PS2_WRITE_CONFBYTE       0x60
#define PS2_AUX_TEST             0xA9
#define PS2_AUX_WRITE            0xD4
#define AUX_DISABLE              0xA7 /* disable the aux device */
#define AUX_ENABLE               0xA8 /* enable the aux device */

#define AUX_ENABLE_XFER          0xf4 /* enable stream mode transfer */
#define AUX_DISABLE_XFER         0xf5 /* disable stream mode transfer */
#define AUX_RESET                0xFF /* RESET mouse        */
#define AUX_COMPLETE             0xaa /* Mouse complete     */
#define AUX_GET_STATUS           0xe9 /* get mouse state    */
#define AUX_DEV_ID               0xf2 /* get device ID      */
#define AUX_STREAM_MODE          0xea /* Set stream mode    */

// Mouse replies
#define RESEND                  (0xfe) /* resend            */
#define ACK                     (0xfa)
#define NUM_OF_RETRIES             (5)

#define TRY_LIMIT             (100000)
#define TIME_DELAY             (10000)
#define IN_BUFF_FULL             (0x2)
#define OUT_BUFF_FULL            (0x1)

#define DATA_PACKAGE_SIZE          (5)
#define FLAG_INIT              0x1000
#define FLAG_RESET             0x2000

struct data {

	int					flags;
	struct sigevent     event;
	struct packet_abs	   tp;       
	int					state;    /* current state (see smart_input for details) */
};

extern struct sigevent event;

static int egalax_init(input_module_t *module);
static int egalax_devctrl(input_module_t *module,int event,void *ptr);
static int egalax_reset(input_module_t *module);
static int egalax_input(input_module_t *module, int num, void *ptr);
static int egalax_parm(input_module_t *module,int opt,char *optarg);
static int egalax_shutdown(input_module_t *module, int delay);
static int egalax_pulse(message_context_t *ctp, int code, unsigned flags, void *data);

void* inter_wait_cycle( input_module_t *module );
int ps2_init( void );
int aux_reset( void );

input_module_t	egalax = {
	NULL,
	NULL,
	NULL,
	0,
	DEVI_CLASS_ABS | DEVI_MODULE_TYPE_DEVICE | DEVI_MODULE_TYPE_PROTO,
	"egalax",
	__DATE__,
	"R",
	NULL,			// data 
	egalax_init,
	egalax_reset,
	egalax_input,
	NULL,
	NULL, //egalax_pulse,
	egalax_parm,
	egalax_devctrl,
	egalax_shutdown, 
};


/* Description: callback initialisation function; it is called when input module    */
/*              initializes input system                                            */
/* Input      : input_module_t * module - pointer to module descriptor              */
/* Output     : None                                                                */
/* Return     : 0 if OK, otherwise - (-1)                                           */
int egalax_init(input_module_t *module)
{
	struct data *dp = module->data;

	ThreadCtl( _NTO_TCTL_IO, 0 );

	if( !module->data )  {
		if( !(dp = module->data = calloc(sizeof *dp, 1)) ) {
			return (-1); 
		}

		SIGEV_INTR_INIT( &(dp->event) );
		dp->flags = ABSOLUTE | ABS_UNCALIBRATED | FLAG_RESET;
		dp->state = 0;
	}

	ps2_init();
	
	return (0);
}

void* inter_wait_cycle( input_module_t *module )
{
	int id;
	struct data *dp = module->data;

	id = InterruptAttachEvent( AUX_IRQ, &dp->event, _NTO_INTR_FLAGS_END );
	if ( id == -1 ) {
		fprintf(stderr, "Mouse interrupt attach failed\n" );
		exit ( -1 );
	}

	uint8_t data[DATA_PACKAGE_SIZE];
	int count = 0;
	for (;;)
	{
		InterruptWait( 0, NULL );
		data[count++] = in8( PS2_DATA_PORT );

		InterruptUnmask( AUX_IRQ, id );

		if ( count == DATA_PACKAGE_SIZE ) {
			(module->input)( module, DATA_PACKAGE_SIZE, data );
			count = 0;
		}

	}

	return NULL;

}
 
int egalax_devctrl(input_module_t *module, int event, void *ptr)
{
	struct data 	*dp = module->data;
	input_module_t	*down = module->down;

	switch(event) {

		case DEVCTL_GETDEVFLAGS:
			*(unsigned short *)ptr = (dp->flags & FLAGS_GLOBAL);
			break;
		case DEVCTL_GETPTRBTNS:
			*(unsigned long *)ptr = 1L;
			break; 
		case DEVCTL_GETPTRCOORD:
			*(unsigned char *)ptr = '\02';
			break;
	 	case DEVCTL_GETPTRPRESS:
			*(unsigned char *)ptr = (dp->flags & ABS_PRESS_DATA)?1:0;
			break;
	 	case DEVCTL_GETCOORDRNG: {
			struct devctl_coord_range	*range = ptr;
			
			// TODO: look for coord range
			range->min = 0;
			range->max = 4095;
		 
			break;
		}
	 	case DEVCTL_GETPRESSRNG: {
			// for all smartset devices at present the range is 0-255
			struct	devctl_coord_range	*range = ptr;

			range->min = 0;
			range->max = 255;
			break;
		}
		default:
			return (-1);
	}

	return (0);
}

/* Description: this callback funtion is called when the module is linked into the  */
/*              event bus;it is used to set initial module state on the protocol    */
/*              level                                                               */
/* Input      : input_module_t * module - pointer to module descriptor              */
/* Output     : None                                                                */
/* Return     : 0 if OK, otherwise -1                                               */
int egalax_reset(input_module_t * module)
{
	struct data             *dp = module->data;
	input_module_t          *down = module->down;

	if((dp->flags & FLAG_INIT) == 0) {  // if not initialized
		pthread_attr_t    attr;

		aux_reset();

		pthread_attr_init( &attr );
		if ( pthread_create( NULL, &attr, &inter_wait_cycle, module ) != EOK ) {
			fprintf(stderr, "Unable to create interrupt handler thread\n" );
			exit ( -1 );
		}

		dp->flags |= FLAG_INIT;
	}

//	if (dp->flags & FLAG_RESET)
//		aux_reset();

    dp->state = 0;

    return 0;
}

/* Description: main protocol processing function.  It will be called  by the       */
/*              device layer to pass it data to process.  Its job is to interpret   */
/*              the data according to the Elographics touch screen protocol, create */
/*              a data structure, fill it in and hand it off to the  filter layer   */
/*              module above it. The protocol is processed using simple state       */
/*              machine.  State info is kept in module's private data               */
/* Input      : input_module_t * module - pointer to module descriptor              */
/*              int num - number of bytes to process                                */
/*              void * arg - raw data to process                                    */
/* Output     : None                                                                */ 
/* Return     : 0 if OK, otherwise -1                                               */
int egalax_input( input_module_t *module, int num, void *ptr ) 
{
	int                i, enqueue = 0;
	uint8_t           *p = (uint8_t *)ptr;
	struct data       *dp = module->data;
	input_module_t    *up = module->up;
        
	if(verbosity >= 5)
	{
       fprintf( stderr, "Received : ");
       for(i = 0; i < num; ++i)
           fprintf( stderr, "%#x ", (int)*((_uint8 *)ptr + i));
       fprintf( stderr, "\n");
	}

	if( p[0] & 0x01 ) dp->tp.buttons = 4L;
	if( (p[0] & 0x01) == 0 ) dp->tp.buttons = 0;

	dp->tp.y = ( (p[1] & 0x7f) << 7) | (p[2] & 0x7f);
	dp->tp.x = ( (p[3] & 0x7f) << 7) | (p[4] & 0x7f);

	dp->state = 0;
	dp->tp.flags = dp->flags;
	clk_get(&dp->tp.timestamp);

	int ret = (up->input)(up, 1, &dp->tp);  // send data upwards

	if(verbosity >= 5)
		fprintf( stderr, "Translated: x = %d, y = %d; ret = %d (%s)\n", dp->tp.x, dp->tp.y, ret, up->name );

	return (0);
}


/* Description: this is a callback function for command line parameter processing   */
/*              (all valid parameters for device module are listed in smartset.args)*/
/* Input      : input_module_t * module - pointer to module descriptor              */
/*              int opt  - parameter code                                           */
/*              char * optarg - optional parameter value                            */
/* Output     : None                                                                */
/* Return     : 0 if OK, otherwise -1                                               */
int egalax_parm(input_module_t *module, int opt, char *optarg)
{
	struct data *dp = module->data;

	if(verbosity >= 5)
		fprintf(stderr, "(%s): params\n", module->name);

 	switch (opt) {
		case 'R':
			dp->flags &= ~FLAG_RESET;
			break;

 		default:
 			return (-1);
 	}

	return (0);
}


/* Description: this is a callback function which is called when resourse manager   */
/*              is shutting down                                                    */
/* Input      : input_module_t * module - pointer to module descriptor              */
/*              int delay  - program doesn't use this parameter                     */
/* Output     : None                                                                */
/* Return     : 0 if OK, otherwise -1                                               */
/* Comment    : Does nothing for the protocol level                                 */
int egalax_shutdown(input_module_t *module, int delay)
{
	return (0);
}


int drain_input_buffer()
{
	int nTries;

	for (nTries = TRY_LIMIT; nTries && ( in8( PS2_STATUS_REGISTER ) & IN_BUFF_FULL); nTries--) {
		nanospin_ns( TIME_DELAY );
	}

	return (nTries == 0);
}
int drain_output_buffer()
{
	int nTries;

	for (nTries = TRY_LIMIT; nTries && ( in8( PS2_STATUS_REGISTER ) & OUT_BUFF_FULL ); nTries--) {
		nanospin_ns( TIME_DELAY );
		in8( PS2_DATA_PORT );
	}

	return (nTries == 0);
}

int ps2_wait_reply()
{
	int nTries;
	uint8_t status;
	int ret = -1;

	// Poll bit 0 (0x01 mask) of the Status Register ("Input buffer empty/full")
	// until it becomes full, or until your time-out expires
	for ( nTries = TRY_LIMIT; nTries; nTries-- )
	{
		status = in8 ( PS2_STATUS_REGISTER );
		if ( (status & OUT_BUFF_FULL) ) {
			ret = in8( PS2_DATA_PORT );

			break;
		}
		nanospin_ns( TIME_DELAY );
	}

	return ret;
}

int ps2_send_command( uint8_t cmd, int n, uint8_t *reply )
{
	int i;

	drain_output_buffer();
	drain_input_buffer();

	out8( PS2_COMMAND_REGISTER, cmd );

	for ( i = 0; i < n; ++i )
		reply[i] = ps2_wait_reply();

	return i;
}


int ps2_send_data( uint8_t data, int n, uint8_t *reply )
{
	int nTries, i;
	uint8_t status;
	int ret = -1;

	for ( nTries = TRY_LIMIT; nTries; --nTries )
	{
		status = in8 ( PS2_STATUS_REGISTER );
		if ( ( status & IN_BUFF_FULL ) == 0 ) { // Input buffer status (0 = empty, 1 = full)
			out8( PS2_DATA_PORT, data );

			break;
		}
		nanospin_ns( TIME_DELAY );
	}

	if ( nTries == 0 )
		return -1;

	for ( i = 0; i < n; ++i )
		reply[i] = ps2_wait_reply();

	return i;
}

int ps2_init( void )
{
	uint8_t reply[1];
	int ret = 0;

	//aux_send_command( AUX_DISABLE_1PORT, 0, NULL );
	ps2_send_command( AUX_DISABLE, 0, NULL );

	ps2_send_command( PS2_TEST_CONTROLLER, 1, reply );
	if ( reply[0] != PS2_SELFTEST_SUCCESS ) {
		fprintf( stderr, "PS2 controller selftest failed (0x%x)\n", reply[0] );
		ret = -1;
	}

	ps2_send_command( PS2_AUX_TEST, 1, reply );
	if ( reply[0] != 0 ) {
		fprintf( stderr, "PS2 second port test failed (0x%x)\n", reply[0] );
		ret = -1;
	}

	ps2_send_command( PS2_READ_CONFBYTE, 1, reply );
	reply[0] |= 0x3;    // Enable interrupts (two first bits)
	ps2_send_command( PS2_WRITE_CONFBYTE, 0, NULL );
	ps2_send_data( reply[0], 0, NULL );

	// Enable device - we are interested only in mouse
	ps2_send_command( AUX_ENABLE, 0, NULL );

	return ret;
}

int aux_reset( void )
{
	uint8_t egalax_diag[] = { 0x0A, 1, 'A' };
	uint8_t reply[4];
	int ret = 0, i;

	reply[0] = 0;
	ps2_send_command( PS2_AUX_WRITE, 0, NULL );
	if ( ps2_send_data( AUX_RESET, 1, reply ) == -1 || reply[0] != ACK ) {
		fprintf( stderr, "Error while reseting aux device\n" );
		ret = -1;
	}

	reply[0] = 0;
	ps2_send_command( PS2_AUX_WRITE, 0, NULL );
	ps2_send_data( AUX_STREAM_MODE, 1, reply );
	if ( reply[0] != ACK )
		fprintf( stderr, "Error transmitting data to aux device\n" );

	reply[0] = 0;
	ps2_send_command( PS2_AUX_WRITE, 0, NULL );
	ps2_send_data( egalax_diag[0], 1, reply );
	if ( reply[0] != ACK )
		fprintf( stderr, "Error transmitting data to aux device\n" );
	reply[0] = 0;
	ps2_send_command( PS2_AUX_WRITE, 0, NULL );
	ps2_send_data( egalax_diag[1], 1, reply );
	if ( reply[0] != ACK )
		fprintf( stderr, "Error transmitting data to aux device\n" );
	reply[0] = 0;
	ps2_send_command( PS2_AUX_WRITE, 0, NULL );
	ps2_send_data( egalax_diag[2], 4, reply );
	if ( reply[0] != ACK )
		fprintf( stderr, "Error transmitting data to aux device\n" );

	for ( i = 1; i <= 3; ++i )
		if ( reply[i] != egalax_diag[i-1] )
			break;
	if ( i != 4 ) {
		fprintf( stderr, "Error transmitting egalax diag\n" );
		ret = -1;
	}

	reply[0] = 0;
	ps2_send_command( PS2_AUX_WRITE, 0, NULL );
	ps2_send_data( AUX_ENABLE_XFER, 1, reply );
	if ( reply[0] != ACK )
		fprintf( stderr, "Error transmitting data to aux device\n" );

/*	ps2_send_command( PS2_AUX_WRITE, 0, NULL );
	ps2_send_data( 0xE9, 4, reply );
	if ( reply[0] == ACK )
		fprintf( stderr, "status = %#x,%#x,%#x\n", reply[1], reply[2], reply[3] );*/


	return ret;
}
